/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2009 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "solidCylinderCloud.H"

// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::solidCylinder::move(solidCylinder::trackData& td)
{
    td.switchProcessor = false;
    td.keepParticle = true;

    const polyMesh& mesh = cloud().pMesh();
    const polyBoundaryMesh& pbMesh = mesh.boundaryMesh();

    scalar deltaT = mesh.time().deltaT().value();
    scalar tEnd = (1.0 - stepFraction())*deltaT;
    scalar dtMax = tEnd;

    while (td.keepParticle && !td.switchProcessor && tEnd > SMALL)
    {
        if (debug)
        {
            Info<< "Time = " << mesh.time().timeName()
                << " deltaT = " << deltaT
                << " tEnd = " << tEnd
                << " steptFraction() = " << stepFraction() << endl;
        }

        // set the lagrangian time-step
        scalar dt = min(dtMax, tEnd);

        // remember which cell the parcel is in
        // since this will change if a face is hit
        label celli = cell();

        dt *= trackToFace(position() + dt*U_, td);

        tEnd -= dt;
        stepFraction() = 1.0 - tEnd/deltaT;

        cellPointWeight cpw(mesh, position(), celli, face());
        scalar rhoc = td.rhoInterp().interpolate(cpw);
        vector Uc = td.UInterp().interpolate(cpw);
        scalar nuc = td.nuInterp().interpolate(cpw);

        scalar rhop = td.spc().rhop();
        scalar magUr = mag(Uc - U_);

        scalar ReFunc = 1.0;
	scalar factor=1.0;
	scalar Re=magUr*d_/nuc;
	scalar E=3.0;
	scalar Asurfs=(2*E+1)/pow(18*E*E,1/3);
	scalar fshape=(1/3)*pow(1.59,1/2)+(2/3)*pow(Asurfs,1/2);
	scalar Cshape=1+0.7*pow(Asurfs-1,1/2)+2.4*(Asurfs-1);
	
	if (fshape!=0.0){
	scalar factor=Cshape/fshape;
	}

	if ( Re > 0.01)
	{
	
       scalar Re=factor*magUr*d_/nuc;
	ReFunc += 0.15*pow(Re, 0.687);
	
	}
    

        scalar Dc = (24.0*nuc/d_)*ReFunc*(3.0/4.0)*(rhoc/(d_*rhop));
	
        U_ = (U_ + dt*(Dc*Uc + (1.0 - rhoc/rhop)*td.g()))/(1.0 + dt*Dc);

        if (onBoundary() && td.keepParticle)
        {
            if (isType<processorPolyPatch>(pbMesh[patch(face())]))
            {
                td.switchProcessor = true;
            }
        }
    }

    return td.keepParticle;
}


bool Foam::solidCylinder::hitPatch
(
    const polyPatch&,
    solidCylinder::trackData&,
    const label
)
{
    return false;
}


bool Foam::solidCylinder::hitPatch
(
    const polyPatch&,
    int&,
    const label
)
{
    return false;
}


void Foam::solidCylinder::hitProcessorPatch
(
    const processorPolyPatch&,
    solidCylinder::trackData& td
)
{
    td.switchProcessor = true;
}


void Foam::solidCylinder::hitProcessorPatch
(
    const processorPolyPatch&,
    int&
)
{}


void Foam::solidCylinder::hitWallPatch
(
    const wallPolyPatch& wpp,
    solidCylinder::trackData& td
)
{
    vector nw = wpp.faceAreas()[wpp.whichFace(face())];
    nw /= mag(nw);

    scalar Un = U_ & nw;
    vector Ut = U_ - Un*nw;

    if (Un > 0)
    {
        U_ -= (1.0 + td.spc().e())*Un*nw;
    }

    U_ -= td.spc().mu()*Ut;
}


void Foam::solidCylinder::hitWallPatch
(
    const wallPolyPatch&,
    int&
)
{}


void Foam::solidCylinder::hitPatch
(
    const polyPatch&,
    solidCylinder::trackData& td
)
{
    td.keepParticle = false;
}


void Foam::solidCylinder::hitPatch
(
    const polyPatch&,
    int&
)
{}


void Foam::solidCylinder::transformProperties (const tensor& T)
{
    Particle<solidCylinder>::transformProperties(T);
    U_ = transform(T, U_);
}


void Foam::solidCylinder::transformProperties(const vector& separation)
{
    Particle<solidCylinder>::transformProperties(separation);
}


// ************************************************************************* //
